@@ -54,9 +54,9 @@ from PyQt5 import QtCore
 
 from picard import (
     PICARD_APP_NAME,
-    config,
     log,
 )
+from picard.config import get_config
 from picard.const import QUERY_LIMIT
 from picard.const.sys import (
     IS_MACOS,
@@ -75,6 +75,7 @@ from picard.util import (
     emptydir,
     find_best_match,
     format_time,
+    is_absolute_path,
     samefile,
     thread,
     tracknum_from_filename,
@@ -138,7 +139,6 @@ class File(QtCore.QObject, Item):
         self.base_filename = os.path.basename(filename)
         self._state = File.UNDEFINED
         self.state = File.PENDING
-        self.error = None
 
         self.orig_metadata = Metadata()
         self.metadata = Metadata()
@@ -185,8 +185,8 @@ class File(QtCore.QObject, Item):
         if self.state != File.PENDING or self.tagger.stopping:
             return
         if error is not None:
-            self.error = str(error)
             self.state = self.ERROR
+            self.error_append(str(error))
 
             # If loading failed, force format guessing and try loading again
             from picard.formats.util import guess_format
@@ -212,10 +212,11 @@ class File(QtCore.QObject, Item):
                 callback(self, remove_file=True)
                 return
         else:
-            self.error = None
+            self.clear_errors()
             self.state = self.NORMAL
             self._copy_loaded_metadata(result)
         # use cached fingerprint from file metadata
+        config = get_config()
         if not config.setting["ignore_existing_acoustid_fingerprints"]:
             fingerprints = self.metadata.getall('acoustid_fingerprint')
             if fingerprints:
@@ -251,6 +252,7 @@ class File(QtCore.QObject, Item):
             if tag in preserved_tags or tag in PRESERVED_TAGS:
                 saved_metadata[tag] = values
         deleted_tags = self.metadata.deleted_tags
+        images_changed = self.metadata.images != metadata.images
         self.metadata.copy(metadata)
         for info in FILE_INFO_TAGS:
             metadata[info] = self.orig_metadata[info]
@@ -262,12 +264,14 @@ class File(QtCore.QObject, Item):
 
         if acoustid and "acoustid_id" not in metadata.deleted_tags:
             self.metadata["acoustid_id"] = acoustid
-        self.metadata_images_changed.emit()
+        if images_changed:
+            self.metadata_images_changed.emit()
 
     def keep_original_images(self):
-        self.metadata.images = self.orig_metadata.images.copy()
-        self.update(signal=False)
-        self.metadata_images_changed.emit()
+        if self.metadata.images != self.orig_metadata.images:
+            self.metadata.images = self.orig_metadata.images.copy()
+            self.update(signal=False)
+            self.metadata_images_changed.emit()
 
     def has_error(self):
         return self.state == File.ERROR
@@ -304,6 +308,7 @@ class File(QtCore.QObject, Item):
 
     def _save_and_rename(self, old_filename, metadata):
         """Save the metadata."""
+        config = get_config()
         # Check that file has not been removed since thread was queued
         # Also don't save if we are stopping.
         if self.state == File.REMOVED:
@@ -356,8 +361,8 @@ class File(QtCore.QObject, Item):
             return
         old_filename = new_filename = self.filename
         if error is not None:
-            self.error = str(error)
             self.state = File.ERROR
+            self.error_append(str(error))
         else:
             self.filename = new_filename = result
             self.base_filename = os.path.basename(new_filename)
@@ -365,23 +370,26 @@ class File(QtCore.QObject, Item):
             temp_info = {}
             for info in FILE_INFO_TAGS:
                 temp_info[info] = self.orig_metadata[info]
+            images_changed = self.orig_metadata.images != self.new_metadata.images
             # Data is copied from New to Original because New may be
             # a subclass to handle id3v23
+            config = get_config()
             if config.setting["clear_existing_tags"]:
                 self.orig_metadata.copy(self.new_metadata)
             else:
                 self.orig_metadata.update(self.new_metadata)
             # After saving deleted tags should no longer be marked deleted
-            self.new_metadata.clear_deleted()
+            self.metadata.clear_deleted()
             self.orig_metadata.clear_deleted()
             self.orig_metadata.length = length
             self.orig_metadata['~length'] = format_time(length)
             for k, v in temp_info.items():
                 self.orig_metadata[k] = v
-            self.error = None
-            self.clear_pending()
+            self.clear_errors()
+            self.clear_pending(signal=False)
             self._add_path_to_metadata(self.orig_metadata)
-            self.metadata_images_changed.emit()
+            if images_changed:
+                self.metadata_images_changed.emit()
 
             # run post save hook
             run_file_post_save_processors(self)
@@ -402,6 +410,7 @@ class File(QtCore.QObject, Item):
 
     def _script_to_filename(self, naming_format, file_metadata, file_extension, settings=None):
         if settings is None:
+            config = get_config()
             settings = config.setting
         metadata = Metadata()
         if settings["clear_existing_tags"]:
@@ -411,6 +420,8 @@ class File(QtCore.QObject, Item):
             metadata.update(file_metadata)
         (filename, new_metadata) = script_to_filename_with_metadata(
             naming_format, metadata, file=self, settings=settings)
+        if not filename:
+            return None
         # NOTE: the script_to_filename strips the extension away
         ext = new_metadata.get('~extension', file_extension)
         return filename + '.' + ext.lstrip('.')
@@ -434,6 +445,8 @@ class File(QtCore.QObject, Item):
         naming_format = settings['file_naming_format']
         if naming_format:
             new_filename = self._script_to_filename(naming_format, metadata, ext, settings)
+            if not new_filename:
+                new_filename = old_filename
             if not settings['rename_files']:
                 new_filename = os.path.join(os.path.dirname(new_filename), old_filename)
             if not settings['move_files']:
@@ -458,10 +471,11 @@ class File(QtCore.QObject, Item):
     def make_filename(self, filename, metadata, settings=None):
         """Constructs file name based on metadata and file naming formats."""
         if settings is None:
+            config = get_config()
             settings = config.setting
         if settings["move_files"]:
             new_dirname = settings["move_files_to"]
-            if not os.path.isabs(new_dirname):
+            if not is_absolute_path(new_dirname):
                 new_dirname = os.path.normpath(os.path.join(os.path.dirname(filename), new_dirname))
         else:
             new_dirname = os.path.dirname(filename)
@@ -505,7 +519,8 @@ class File(QtCore.QObject, Item):
             return
         counters = defaultdict(lambda: 0)
         images = []
-        if config.setting["caa_save_single_front_image"]:
+        config = get_config()
+        if config.setting["save_only_one_front_image"]:
             front = metadata.images.get_front_image()
             if front:
                 images.append(front)
@@ -521,6 +536,7 @@ class File(QtCore.QObject, Item):
         if new_path == old_path:
             # skip, same directory, nothing to move
             return
+        config = get_config()
         patterns = config.setting["move_additional_files_pattern"]
         pattern_regexes = set()
         for pattern in patterns.split():
@@ -572,11 +588,13 @@ class File(QtCore.QObject, Item):
             log.debug("Moving %r from %r to %r", self, self.parent, parent)
             self.clear_lookup_task()
             self.tagger._acoustid.stop_analyze(self)
+            new_album = True
             if self.parent:
+                new_album = self.parent.album != parent.album
                 self.clear_pending()
-                self.parent.remove_file(self)
+                self.parent.remove_file(self, new_album=new_album)
             self.parent = parent
-            self.parent.add_file(self)
+            self.parent.add_file(self, new_album=new_album)
             self.acoustid_update()
 
     def _move(self, parent):
@@ -619,6 +637,7 @@ class File(QtCore.QObject, Item):
         new_metadata = self.new_metadata
         names = set(new_metadata.keys())
         names.update(self.orig_metadata.keys())
+        config = get_config()
         clear_existing_tags = config.setting["clear_existing_tags"]
         ignored_tags = config.setting["compare_ignore_tags"]
         for name in names:
@@ -741,6 +760,7 @@ class File(QtCore.QObject, Item):
             if lookuptype == File.LOOKUP_ACOUSTID:
                 threshold = 0
             else:
+                config = get_config()
                 threshold = config.setting['file_lookup_threshold']
 
             trackmatch = self._match_to_track(tracks, threshold=threshold)
@@ -816,16 +836,19 @@ class File(QtCore.QObject, Item):
     def set_pending(self):
         if self.state != File.REMOVED:
             self.state = File.PENDING
-            self.update_item()
+            self.update_item(update_selection=False)
 
-    def clear_pending(self):
+    def clear_pending(self, signal=True):
         if self.state == File.PENDING:
-            self.state = File.NORMAL if self.similarity == 1.0 else File.CHANGED
-            self.update_item()
+            self.state = File.NORMAL
+            # Update file to recalculate changed state
+            self.update(signal=False)
+            if signal:
+                self.update_item(update_selection=False)
 
-    def update_item(self):
+    def update_item(self, update_selection=True):
         if self.item:
-            self.item.update()
+            self.item.update(update_selection=update_selection)
 
     def iterfiles(self, save=False):
         yield self
