@@ -9,6 +9,7 @@
 # Copyright (C) 2014 Sophist-UK
 # Copyright (C) 2016, 2018 Sambhav Kothari
 # Copyright (C) 2018 Wieland Hoffmann
+# Copyright (C) 2021 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -34,13 +35,17 @@ from PyQt5 import (
     QtWidgets,
 )
 
-from picard import (
-    config,
-    log,
+from picard import log
+from picard.config import (
+    IntOption,
+    get_config,
 )
 from picard.util import reconnect
 
-from picard.ui import PicardDialog
+from picard.ui import (
+    FONT_FAMILY_MONOSPACE,
+    PicardDialog,
+)
 from picard.ui.colors import interface_colors
 
 
@@ -152,7 +157,7 @@ class VerbosityMenu(QtWidgets.QMenu):
 class LogView(LogViewCommon):
 
     options = [
-        config.IntOption("setting", "log_verbosity", log.VERBOSITY_DEFAULT),
+        IntOption("setting", "log_verbosity", log.VERBOSITY_DEFAULT),
     ]
 
     def __init__(self, parent=None):
@@ -205,6 +210,8 @@ class LogView(LogViewCommon):
         self.hbox.addWidget(self.save_log_as_button)
         self.save_log_as_button.clicked.connect(self._save_log_as_do)
 
+        self._prev_logitem_level = log.VERBOSITY_DEFAULT
+
     def _clear_highlight_do(self):
         self.highlight_text.setText('')
         self.highlight_button.setEnabled(False)
@@ -232,11 +239,9 @@ class LogView(LogViewCommon):
     def _setup_formats(self):
         interface_colors.load_from_config()
         self.formats = {}
-        font = QtGui.QFont()
-        font.setFamily("Monospace")
         for level, feat in log.levels_features.items():
             text_fmt = QtGui.QTextCharFormat()
-            text_fmt.setFont(font)
+            text_fmt.setFontFamily(FONT_FAMILY_MONOSPACE)
             text_fmt.setForeground(interface_colors.get_qcolor(feat.color_key))
             self.formats[level] = text_fmt
 
@@ -292,10 +297,10 @@ class LogView(LogViewCommon):
     def _add_entry(self, logitem):
         if not self.is_shown(logitem):
             return
-        fmt = self.textCursor.blockCharFormat()
-        self.textCursor.setBlockCharFormat(self._format(logitem.level))
+        if self._prev_logitem_level != logitem.level:
+            self.textCursor.setBlockCharFormat(self._format(logitem.level))
+            self._prev_logitem_level = logitem.level
         super()._add_entry(logitem)
-        self.textCursor.setBlockCharFormat(fmt)
 
     def _set_verbosity(self, level):
         self.verbosity = level
@@ -303,6 +308,7 @@ class LogView(LogViewCommon):
 
     def _verbosity_changed(self, level):
         if level != self.verbosity:
+            config = get_config()
             config.setting['log_verbosity'] = level
             QtCore.QObject.tagger.set_log_level(level)
             self.verbosity = level
